--# Copyright (C) @BABOLO  2002..2008 http://www.babolo.ru/
--# Copyright (C) 2005 Michael Deryugin <dem@pikenet.ru>
--# Copyright (C) GD 2005..2007 <gd@pikenet.ru>
--# All rights reserved.
--#
--# Redistribution and use in source and binary forms, with or without
--# modification, are permitted provided that the following conditions
--# are met:
--# 1. Redistributions of source code must retain the above copyright
--#    notice, this list of conditions and the following disclaimer.
--# 2. Redistributions in binary form must reproduce the above copyright
--#    notice, this list of conditions and the following disclaimer in the
--#    documentation and/or other materials provided with the distribution.
--#
--# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
--# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
--# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
--# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
--# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
--# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
--# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
--# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
--# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
--# SUCH DAMAGE.
--#
--# $Id: pglib_bit.sql.m4,v 1.1 2008/01/23 23:47:54 babolo Exp $

--#          --
--#-------------------------------------------------------------
CREATE OR REPLACE FUNCTION num2bit(integer) RETURNS varbit
 AS '$libdir/pglib_bit.so.VMAJOR', 'num2bit'
 LANGUAGE C IMMUTABLE STRICT
;
CREATE OR REPLACE FUNCTION bit2num(varbit) RETURNS SETOF int4
 AS '$libdir/pglib_bit.so.VMAJOR', 'bit2num'
 LANGUAGE C IMMUTABLE STRICT
;
COMMENT ON FUNCTION bit2num(varbit)       IS '   ,        ,       '
;


--#    --
--#----------------------------
CREATE OR REPLACE FUNCTION bitex0(bit, int4) RETURNS varbit
 AS '$libdir/pglib_bit.so.VMAJOR', 'bitex0'
 LANGUAGE C IMMUTABLE STRICT
;
COMMENT ON FUNCTION bitex0(bit, int4) IS '   ,    0';

CREATE OR REPLACE FUNCTION bitex1(bit, int4) RETURNS varbit
 AS '$libdir/pglib_bit.so.VMAJOR', 'bitex1'
 LANGUAGE C IMMUTABLE STRICT
;
COMMENT ON FUNCTION bitex1(bit, int4) IS '   ,    1';



--#  varbit  bool  AND  OR --
--#-------------------------------------
CREATE OR REPLACE FUNCTION boolor(bit) RETURNS bool
 AS '$libdir/pglib_bit.so.VMAJOR', 'boolor'
 LANGUAGE C IMMUTABLE STRICT
;
COMMENT ON FUNCTION boolor(bit)       IS 'OR    ';

CREATE OR REPLACE FUNCTION booland(bit) RETURNS bool
 AS '$libdir/pglib_bit.so.VMAJOR', 'booland'
 LANGUAGE C IMMUTABLE STRICT
;
COMMENT ON FUNCTION booland(bit)      IS 'AND    ';



--#    , ,            --
--#    ,    --
--#----------------------------------------------------------------------
CREATE OR REPLACE FUNCTION public.varbiteq(bit, bit) RETURNS bool
 AS '$libdir/pglib_bit.so.VMAJOR', 'varbiteq'
 LANGUAGE C IMMUTABLE STRICT
;
COMMENT ON FUNCTION varbiteq(bit, bit)   IS '    ';

CREATE OR REPLACE FUNCTION varbitand(bit, bit) RETURNS varbit
 AS '$libdir/pglib_bit.so.VMAJOR', 'varbitand'
 LANGUAGE C IMMUTABLE STRICT
;
COMMENT ON FUNCTION varbitand(bit, bit)  IS 'AND      ';

CREATE OR REPLACE FUNCTION varbitor(bit, bit) RETURNS varbit
 AS '$libdir/pglib_bit.so.VMAJOR', 'varbitor'
 LANGUAGE C IMMUTABLE STRICT
;
COMMENT ON FUNCTION varbitor(bit, bit)   IS 'OR      ';

CREATE OR REPLACE FUNCTION varbitxor(bit, bit) RETURNS varbit
 AS '$libdir/pglib_bit.so.VMAJOR', 'varbitxor'
 LANGUAGE C IMMUTABLE STRICT
;
COMMENT ON FUNCTION varbitxor(bit, bit)  IS 'XOR      ';

CREATE OR REPLACE FUNCTION varbitrst(bit, bit) RETURNS varbit
 AS '$libdir/pglib_bit.so.VMAJOR', 'varbitrst'
 LANGUAGE C IMMUTABLE STRICT
;
COMMENT ON FUNCTION varbitrst(bit, bit)  IS '$1 AND ~$2,       ';

CREATE OR REPLACE FUNCTION varbitmeq(bit, bit, bit) RETURNS bool
 AS '$libdir/pglib_bit.so.VMAJOR', 'varbitmeq'
 LANGUAGE C IMMUTABLE STRICT
;
COMMENT ON FUNCTION varbitmeq(bit, bit, bit)  IS '$1  $2    $3';

CREATE OPERATOR ==
     ( LEFTARG = bit
     , RIGHTARG = bit
     , PROCEDURE = public.varbiteq
     , COMMUTATOR = ==
     )
;
COMMENT ON OPERATOR == (bit, bit)        IS '    ';

CREATE OPERATOR /|
     ( LEFTARG = bit
     , RIGHTARG = bit
     , PROCEDURE = varbitand
     , COMMUTATOR = /|
     )
;
COMMENT ON OPERATOR /| (bit, bit)        IS 'AND      ';

CREATE OPERATOR |/
     ( LEFTARG = bit
     , RIGHTARG = bit
     , PROCEDURE = varbitor
     , COMMUTATOR = |/
     )
;
COMMENT ON OPERATOR |/ (bit, bit)        IS 'OR      ';

CREATE OPERATOR ^
     ( LEFTARG = bit
     , RIGHTARG = bit
     , PROCEDURE = varbitxor
     , COMMUTATOR = ^
     )
;
COMMENT ON OPERATOR ^ (bit, bit)         IS 'XOR      ';

CREATE OPERATOR !
     ( LEFTARG = bit
     , RIGHTARG = bit
     , PROCEDURE = varbitrst
     )
;
COMMENT ON OPERATOR ! (bit, bit)         IS '$1 AND ~$2,       ';



--# / varbit --
--#---------------------
CREATE OR REPLACE FUNCTION varbit(text) RETURNS varbit
 AS 'DECLARE
        i int;
        r varbit;
        c char;
     BEGIN
        r := ''''::varbit;
        FOR i IN 1..length($1) LOOP
           c := substr($1, i, 1);
           IF c = ''1''
           THEN r := varbitor(r, bitex0(''1''::varbit, i) >> (i - 1));
           ELSIF c <> ''0''
           THEN RAISE EXCEPTION ''"%" is not a valid binary digit'', c;
           END IF;
        END LOOP;
        RETURN r;
     END;
    '
 LANGUAGE 'plpgsql' IMMUTABLE STRICT
;
COMMENT ON FUNCTION varbit(text) IS '   varbit';

CREATE OR REPLACE FUNCTION text(varbit) RETURNS text 
 AS 'DECLARE
        i int;
        t text;
     BEGIN
        t := ''''::text;
        FOR i IN 1..bit_length($1) LOOP
           IF boolor(varbitand($1, (bitex0(''1''::varbit, i) >> (i - 1))))
           THEN t := t || ''1'';
           ELSE t := t || ''0'';
           END IF;
        END LOOP;
        RETURN t;
     END;
    '
 LANGUAGE 'plpgsql' IMMUTABLE STRICT
;
COMMENT ON FUNCTION varbit(text) IS ' varbit  ';



--#  bool  AND  OR --
--#-------------------------------
CREATE OR REPLACE FUNCTION bool_and(bool, bool) RETURNS bool
 AS 'SELECT $1 AND $2'
 LANGUAGE SQL IMMUTABLE STRICT
;
COMMENT ON FUNCTION bool_and(bool, bool) IS '        ';

CREATE OR REPLACE FUNCTION bool_or(bool, bool) RETURNS bool
 AS 'SELECT $1 OR $2'
 LANGUAGE SQL IMMUTABLE STRICT
;
COMMENT ON FUNCTION bool_or(bool, bool)  IS '        ';

CREATE AGGREGATE conjunction
     ( BASETYPE = bool
     , SFUNC = bool_and
     , STYPE = bool
     , INITCOND = 1
     )
;
COMMENT ON AGGREGATE conjunction(bool)   IS ' AND  bool';

CREATE AGGREGATE disjunction
     ( BASETYPE = bool
     , SFUNC = bool_or
     , STYPE = bool
     , INITCOND = 0
     )
;
COMMENT ON AGGREGATE disjunction(bool)   IS ' OR  bool';

--#  varbit  AND  OR --
--#---------------------------------
CREATE OR REPLACE FUNCTION varbitandn(bit, bit) RETURNS varbit
 AS '$libdir/pglib_bit.so.VMAJOR', 'varbitand'
 LANGUAGE C IMMUTABLE
;
COMMENT ON FUNCTION varbitandn(bit, bit) IS 'varbitand  ';

CREATE OR REPLACE FUNCTION varbitorn(bit, bit) RETURNS varbit
 AS '$libdir/pglib_bit.so.VMAJOR', 'varbitor'
 LANGUAGE C IMMUTABLE
;
COMMENT ON FUNCTION varbitorn(bit, bit)  IS 'varbitor  ';

CREATE AGGREGATE conjunction
     ( BASETYPE = bit
     , SFUNC = varbitandn
     , STYPE = varbit
     )
;
COMMENT ON AGGREGATE conjunction(bit)    IS ' AND  bit';

CREATE AGGREGATE disjunction
     ( BASETYPE = bit
     , SFUNC = varbitorn
     , STYPE = varbit
     )
;
COMMENT ON AGGREGATE disjunction(bit)    IS ' OR  bit';




--# ? ,   ? --
--#------------------------
CREATE OPERATOR /|/
     ( LEFTARG = bit
     , RIGHTARG = bit
     , PROCEDURE = varbitxor
     , COMMUTATOR = /|/
     )
;
